Package: memes
Type: Package
Title: motif matching, comparison, and de novo discovery using the MEME
        Suite
Version: 1.20.0
Authors@R: person("Spencer", "Nystrom",
    email = "nystromdev@gmail.com", 
    role = c("aut", "cre", "cph"),
    comment = c(ORCID = "0000-0003-1000-1579"))
Description: A seamless interface to the MEME Suite family of tools for motif
    analysis. 'memes' provides data aware utilities for using GRanges objects as
    entrypoints to motif analysis, data structures for examining & editing motif
    lists, and novel data visualizations. 'memes' functions and data structures are
    amenable to both base R and tidyverse workflows.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: Biostrings, dplyr, cmdfun (>= 1.0.2), GenomicRanges, ggplot2,
        magrittr, matrixStats, methods, patchwork, processx, purrr,
        rlang, readr, stats, tools, tibble, tidyr, utils, usethis,
        universalmotif (>= 1.9.3), xml2
Suggests: cowplot, BSgenome.Dmelanogaster.UCSC.dm3,
        BSgenome.Dmelanogaster.UCSC.dm6, forcats, testthat (>= 2.1.0),
        knitr, MotifDb, pheatmap, PMCMRplus, plyranges (>= 1.9.1),
        rmarkdown, covr
biocViews: DataImport, FunctionalGenomics, GeneRegulation,
        MotifAnnotation, MotifDiscovery, SequenceMatching, Software
RoxygenNote: 7.3.2
SystemRequirements: Meme Suite (v5.3.3 or above)
        <http://meme-suite.org/doc/download.html>
VignetteBuilder: knitr
URL: https://snystrom.github.io/memes/,
        https://github.com/snystrom/memes
BugReports: https://github.com/snystrom/memes/issues
Depends: R (>= 4.1)
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/memes
git_branch: RELEASE_3_23
git_last_commit: 8fc94ef
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:43:14 UTC; biocbuild
Author: Spencer Nystrom [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-1000-1579>)
Maintainer: Spencer Nystrom <nystromdev@gmail.com>
Built: R 4.6.0; ; 2026-04-29 04:49:22 UTC; unix
