# Generated by roxygen2: do not edit by hand

S3method(get_sequence,CompressedGRangesList)
S3method(get_sequence,GRanges)
S3method(get_sequence,GRangesList)
S3method(get_sequence,GenomicRangesList)
S3method(get_sequence,SimpleGRangesList)
S3method(get_sequence,character)
S3method(get_sequence,data.frame)
S3method(get_sequence,list)
S3method(has_duplicate_motifs,data.frame)
S3method(has_duplicate_motifs,list)
S3method(has_duplicate_motifs,universalmotif_df)
S3method(motif_input,character)
S3method(motif_input,data.frame)
S3method(motif_input,list)
S3method(motif_input,universalmotif)
S3method(motif_input,universalmotif_df)
S3method(plot_sequence_heatmap,default)
S3method(plot_sequence_heatmap,list)
S3method(remove_duplicate_motifs,data.frame)
S3method(remove_duplicate_motifs,list)
S3method(remove_duplicate_motifs,universalmotif_df)
S3method(runAme,BStringSetList)
S3method(runAme,default)
S3method(runAme,list)
S3method(runDreme,BStringSetList)
S3method(runDreme,default)
S3method(runDreme,list)
S3method(runMeme,BStringSetList)
S3method(runMeme,default)
S3method(runMeme,list)
S3method(runStreme,BStringSet)
S3method(runStreme,BStringSetList)
S3method(runStreme,default)
S3method(runStreme,list)
S3method(runTomTom,default)
S3method(runTomTom,list)
S3method(sequence_as_granges,BStringSet)
S3method(sequence_as_granges,DNAStringSet)
S3method(sequence_as_granges,RNAStringSet)
S3method(sequence_input,AAStringSet)
S3method(sequence_input,BStringSet)
S3method(sequence_input,DNAStringSet)
S3method(sequence_input,character)
export(add_sequence)
export(ame_compare_heatmap_methods)
export(check_meme_install)
export(drop_best_match)
export(force_best_match)
export(get_sequence)
export(has_duplicate_motifs)
export(importAme)
export(importDremeXML)
export(importFimo)
export(importMeme)
export(importStremeXML)
export(importTomTomXML)
export(meme_is_installed)
export(nest_tomtom)
export(plot_ame_heatmap)
export(plot_sequence_heatmap)
export(remove_duplicate_motifs)
export(runAme)
export(runDreme)
export(runFimo)
export(runMeme)
export(runStreme)
export(runTomTom)
export(update_best_match)
export(view_tomtom_hits)
export(write_fasta)
importFrom(Biostrings,BStringSetList)
importFrom(Biostrings,getSeq)
importFrom(Biostrings,writeXStringSet)
importFrom(GenomicRanges,`mcols<-`)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,mcols)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(dplyr,desc)
importFrom(ggplot2,.pt)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_continuous)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_ecdf)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,"%T>%")
importFrom(matrixStats,colMaxs)
importFrom(methods,is)
importFrom(readr,cols)
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,enquo)
importFrom(rlang,sym)
importFrom(stats,reorder)
importFrom(tibble,rowid_to_column)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,nest)
importFrom(tidyr,separate)
importFrom(tidyr,unnest)
importFrom(utils,compareVersion)
importFrom(utils,head)
